#include <windows.h>
#include <windowsx.h>
#include "gen.h"
#include "resource.h"
#include <stdio.h>
#include <winsock.h>    /* for socket(),... */

WNDPROC lpWndProcOld;

void config();
void quit();
int init();
void MakeHTTPRequest(char * URL);
char * EncodeURL(char *);
VOID CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime);

#define PLUGIN_DESC "Now Playing?"
#define TIMER_ID 322
#define TIMEOUT 1000

char scriptaddr[512];
char enabledstr[4];
char old_title_string[2048];
char iniFile[MAX_PATH];

winampGeneralPurposePlugin plugin =
{
	GPPHDR_VER,
	PLUGIN_DESC " v0.3",
	init,
	config,
	quit,
};

BOOL CALLBACK configDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch (uMsg) {	
	case WM_CLOSE:
		EndDialog(hDlg,TRUE);
		return 0;
	case WM_INITDIALOG:
		if (!strcmp(enabledstr,"Yes")) CheckDlgButton(hDlg,IDC_ENABLED,BST_CHECKED);
		SetDlgItemText(hDlg,IDC_URL,scriptaddr);
		
		break;
	case WM_COMMAND:
		switch (GET_WM_COMMAND_ID(wParam, lParam)) {
		case IDOK:
			strcpy(enabledstr,(IsDlgButtonChecked(hDlg, IDC_ENABLED) == BST_CHECKED) ? "Yes" : "No");
			KillTimer(plugin.hwndParent,TIMER_ID);
			if (IsDlgButtonChecked(hDlg, IDC_ENABLED) == BST_CHECKED)
				SetTimer(plugin.hwndParent,TIMER_ID,TIMEOUT,TimerProc);

			GetDlgItemText(hDlg,IDC_EDIT1,scriptaddr,sizeof(scriptaddr)-1);
			
			WritePrivateProfileString(PLUGIN_DESC,"URL",scriptaddr,iniFile);
			WritePrivateProfileString(PLUGIN_DESC,"Enabled",enabledstr,iniFile);

			old_title_string[0]='\0'; // force update
		case IDCANCEL:
			EndDialog(hDlg,TRUE);
			break;
		}
	default:
		return 0;
	}

	return 1;
}

void config()
{
	DialogBox(plugin.hDllInstance, (const char *)IDD_CONFIG, plugin.hwndParent, configDlgProc);
}

void quit()
{
	char URL[2048];
	KillTimer(plugin.hwndParent,TIMER_ID);
	if (!strcmp(enabledstr,"Yes")) {
		sprintf(URL,scriptaddr,"Nothing");
		MakeHTTPRequest(URL);
	}
}

VOID CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime) {
	char URL[2048],*encoded_title,this_title[2048],title_string[2048],*p;
	int status;
	if (idEvent == TIMER_ID) {
		// in case this take a while don't trigger before we're done
		KillTimer(plugin.hwndParent,TIMER_ID);
		
		// get winamp status
		status=SendMessage(hwnd,WM_USER,0,104);
		
		// get track title
		GetWindowText(hwnd,this_title,sizeof(this_title));
		p=strstr(this_title," - Winamp");
		if (!p) sprintf(title_string,"Nothing");	
		else {
			p[0]='\0';
			switch (status) {
			case 1:
				sprintf(title_string,"%s",strstr(this_title,". ")+2);
				break;
			case 3:
				sprintf(title_string,"%s [Paused]",strstr(this_title,". ")+2);
				break;
			default:
				sprintf(title_string,"%s [Stopped]",strstr(this_title,". ")+2);
			}
		}
			
		if (strcmp(title_string,old_title_string)) {
    		
			// output name
			//MessageBox(hwnd,title_string,"new song",MB_OK);
			encoded_title = EncodeURL(title_string);
			sprintf(URL,scriptaddr,encoded_title);
			free(encoded_title); encoded_title=0;
			MakeHTTPRequest(URL);
		}
		strcpy(old_title_string,title_string);

		SetTimer(plugin.hwndParent,TIMER_ID,TIMEOUT,TimerProc);
	}
}

__declspec( dllexport ) winampGeneralPurposePlugin * winampGetGeneralPurposePlugin()
{
	return &plugin;
}

char * EncodeURL(char * url) {
	char * r = malloc(strlen(url)*3);
	unsigned int c,c2;

	for (c=0,c2=0; c < strlen(url); c++) {
		if (!((url[c] >= 'A' && url[c] <= 'Z') || (url[c] >= 'a' && url[c] <= 'z') ||
			(url[c] >= '0' && url[c] <= '9'))) {
			sprintf(r+c2,"%%%02x",url[c]);
			c2+=2;
		} else r[c2]=url[c];
		c2++;
	}
	r[c2]='\0';
	return r;
}

void MakeHTTPRequest(char *URL)
{
	WORD wVersionRequested = MAKEWORD(1,1);
	WSADATA wsaData;
	int nRet;
	unsigned int c,c2;
	short nPort;
    LPHOSTENT lpHostEntry;
	char szBuf[1024];
	char szServer[1024];
	//char szPath[1024];
	char szPort[10];
	SOCKET	theSocket;
	SOCKADDR_IN saServer;

	//MessageBox(NULL,URL,"request",MB_OK);
	//return;
	
	nPort = 80;

	// parse URL
	if (strstr(URL,"http://")==URL) c=7;
	else c=0;
	for (c2=0; c < strlen(URL) && URL[c] != '/' && URL[c] != ':'; c++,c2++) szServer[c2]=URL[c];
	szServer[c2]='\0';
	if (URL[c]==':') {
		c++;
		for (c2=0; c < strlen(URL) && URL[c] != '/'; c++,c2++) szPort[c2]=URL[c];
		szPort[c2]='\0';
		sscanf(szPort,"%i",&nPort);
	}
	/*for (c2=0; c < strlen(URL); c++,c2++) szPath[c2]=URL[c];
	szPath[c2]='\0';*/

	//
	// Initialize WinSock and check the version
	//
	nRet = WSAStartup(wVersionRequested, &wsaData);
	if (wsaData.wVersion != wVersionRequested)
	{	
		return;
	}

	//
	// Find the server
	//

	lpHostEntry = gethostbyname(szServer);
    if (lpHostEntry == NULL)
    {
        //PRINTERROR("gethostbyname()");
        return;
    }

	//
	// Create a TCP/IP stream socket
	//

	theSocket = socket(AF_INET,				// Address family
					   SOCK_STREAM,			// Socket type
					   IPPROTO_TCP);		// Protocol
	if (theSocket == INVALID_SOCKET)
	{
		//PRINTERROR("socket()");
		return;
	}

	//
	// Fill in the address structure
	//

	saServer.sin_family = AF_INET;
	saServer.sin_addr = *((LPIN_ADDR)*lpHostEntry->h_addr_list);
										// ^ Server's address
	saServer.sin_port = htons(nPort);	// Port number

	//
	// connect to the server
	//

	nRet = connect(theSocket,				// Socket
				   (LPSOCKADDR)&saServer,	// Server address
				   sizeof(struct sockaddr));// Length of server address structure
	if (nRet == SOCKET_ERROR)
	{
		//PRINTERROR("socket()");
		closesocket(theSocket);
		return;
	}


	//
	// Send data to the server
	//
	
	// use full URL for servers with multiple domains on one IP
	sprintf(szBuf,"GET %s HTTP/1.0\n\n",URL);
	nRet = send(theSocket,				// Connected socket
				szBuf,					// Data buffer
				strlen(szBuf),			// Length of data
				0);						// Flags
	if (nRet == SOCKET_ERROR)
	{
		closesocket(theSocket);
		return;
	}

	closesocket(theSocket);

	WSACleanup();

}

int init()
{
	if (GetModuleFileName(GetModuleHandle(NULL), iniFile, MAX_PATH)) {
		char * lastSlash = strrchr(iniFile, '\\');
		*(lastSlash + 1) = 0;
		strcat(iniFile, "winamp.ini");
	}

	GetPrivateProfileString(PLUGIN_DESC,"URL","http://www.example.bar/script.php?%s",scriptaddr,sizeof(scriptaddr),iniFile);
	GetPrivateProfileString(PLUGIN_DESC,"Enabled","No",enabledstr,sizeof(enabledstr),iniFile);
	
	old_title_string[0]='\0';

	if (!strcmp(enabledstr,"Yes")) {
		KillTimer(plugin.hwndParent,TIMER_ID);
		SetTimer(plugin.hwndParent,TIMER_ID,TIMEOUT,TimerProc);
	}
	
	return 0;
}
